@@ -5,7 +5,7 @@
 #define STRINGIFY(x) #x
 #define TOSTRING(x) STRINGIFY(x)
 
-#define PCONVERT_VERSION "0.3.10"
+#define PCONVERT_VERSION "0.3.11"
 #define PCONVERT_COMPILATION_DATE __DATE__
 #define PCONVERT_COMPILATION_TIME __TIME__
 
@@ -25,6 +25,24 @@
 
 #define PCONVERT_PLATFORM_CPU_BITS sizeof(void *) * 8
 
+#ifdef PCONVERT_OPENCL
+#define PCONVERT_OPENCL_V 1
+#define PCONVERT_OPENCL_S " opencl"
+#else
+#define PCONVERT_OPENCL_V 0
+#define PCONVERT_OPENCL_S ""
+#endif
+
+#ifdef PCONVERT_EXTENSION
+#define PCONVERT_EXTENSION_V 1
+#define PCONVERT_EXTENSION_S " python"
+#else
+#define PCONVERT_EXTENSION_V 0
+#define PCONVERT_EXTENSION_S ""
+#endif
+
+#define PCONVERT_FEATURES "cpu" PCONVERT_OPENCL_S PCONVERT_EXTENSION_S
+
 #define PCONVERT_ALGORITHMS "multiplicative", "source_over", "destination_over",\
     "mask_top", "first_top", "first_bottom", "disjoint_over", "disjoint_under", "disjoint_debug"
 
@@ -42,6 +60,9 @@
 #define RAISE_S(...) abort_(__VA_ARGS__); return ERROR
 #define RAISE_M(message) set_last_error(message); return ERROR
 #define RAISE_F(message, ...) set_last_error_f(message, __VA_ARGS__); return ERROR
+#define EXCEPT_P(input) if(IS_ERROR(input)) {\
+    print_(last_error == NULL ? "Unknown error" : last_error);\
+} while(FALSE)
 #define EXCEPT_S(input) if(IS_ERROR(input)) {\
     print_(last_error == NULL ? "Unknown error" : last_error);\
     return ERROR;\
