@@ -655,7 +655,7 @@ ERROR_T duplicate_image(struct pcv_image *origin, struct pcv_image *target) {
     png_uint_32 row_size = (png_uint_32) png_get_rowbytes(origin->png_ptr, origin->info_ptr);
     png_uint_32 buffer_size = row_size * origin->height;
     png_byte *buffer = (png_byte *) malloc(buffer_size);
-    copy_image(origin, target);
+    VALIDATE(copy_image(origin, target));
     target->rows = (png_bytep *) malloc(rows_size);
     memcpy(buffer, *origin->rows, buffer_size);
     for(y = 0; y < origin->height; y++) {
@@ -690,20 +690,30 @@ ERROR_T compose_images_extra(
     char name[1024];
     struct pcv_image bottom, top, final;
     char demultiply = is_multiplied(algorithm);
-    read_png(join_path(base_path, "sole.png", path), demultiply, &bottom);
-    read_png(join_path(base_path, "back.png", path), demultiply, &top);
-    blend_images_extra(&bottom, &top, algorithm, params, use_opencl); release_image(&top);
-    read_png(join_path(base_path, "front.png", path), demultiply, &top);
-    blend_images_extra(&bottom, &top, algorithm, params, use_opencl); release_image(&top);
-    read_png(join_path(base_path, "shoelace.png", path), demultiply, &top);
-    blend_images_extra(&bottom, &top, algorithm, params, use_opencl); release_image(&top);
+
+    VALIDATE(read_png(join_path(base_path, "sole.png", path), demultiply, &bottom));
+    VALIDATE(read_png(join_path(base_path, "back.png", path), demultiply, &top));
+    VALIDATE(blend_images_extra(&bottom, &top, algorithm, params, use_opencl));
+    VALIDATE(release_image(&top));
+
+    VALIDATE(read_png(join_path(base_path, "front.png", path), demultiply, &top));
+    VALIDATE(blend_images_extra(&bottom, &top, algorithm, params, use_opencl));
+    VALIDATE(release_image(&top));
+
+    VALIDATE(read_png(join_path(base_path, "shoelace.png", path), demultiply, &top));
+    VALIDATE(blend_images_extra(&bottom, &top, algorithm, params, use_opencl));
+    VALIDATE(release_image(&top));
+
     if(demultiply) { multiply_image(&bottom); }
+
     sprintf(name, "background_%s.png", background);
-    read_png(join_path(base_path, name, path), FALSE, &final);
-    blend_images_extra(&final, &bottom, algorithm, params, use_opencl); release_image(&bottom);
+    VALIDATE(read_png(join_path(base_path, name, path), FALSE, &final));
+    VALIDATE(blend_images_extra(&final, &bottom, algorithm, params, use_opencl));
+    VALIDATE(release_image(&bottom));
+
     sprintf(name, "result_%s_%s.png", algorithm, background);
-    write_png_extra(&final, FALSE, join_path(base_path, name, path), compression, filter);
-    release_image(&final);
+    VALIDATE(write_png_extra(&final, FALSE, join_path(base_path, name, path), compression, filter));
+    VALIDATE(release_image(&final));
 
     NORMAL;
 }
@@ -711,42 +721,42 @@ ERROR_T compose_images_extra(
 ERROR_T pcompose(int argc, char **argv) {
     if(argc != 3) { RAISE_M("Usage: pconvert compose <directory>"); }
 
-    compose_images(argv[2], "alpha", NULL, "alpha");
-    compose_images(argv[2], "alpha", NULL, "white");
-    compose_images(argv[2], "alpha", NULL, "blue");
-    compose_images(argv[2], "alpha", NULL, "texture");
-    compose_images(argv[2], "multiplicative", NULL, "alpha");
-    compose_images(argv[2], "multiplicative", NULL, "white");
-    compose_images(argv[2], "multiplicative", NULL, "blue");
-    compose_images(argv[2], "multiplicative", NULL, "texture");
-    compose_images(argv[2], "source_over", NULL, "alpha");
-    compose_images(argv[2], "source_over", NULL, "white");
-    compose_images(argv[2], "source_over", NULL, "blue");
-    compose_images(argv[2], "source_over", NULL, "texture");
-    compose_images(argv[2], "destination_over", NULL, "alpha");
-    compose_images(argv[2], "destination_over", NULL, "white");
-    compose_images(argv[2], "destination_over", NULL, "blue");
-    compose_images(argv[2], "destination_over", NULL, "texture");
-    compose_images(argv[2], "first_top", NULL, "alpha");
-    compose_images(argv[2], "first_top", NULL, "white");
-    compose_images(argv[2], "first_top", NULL, "blue");
-    compose_images(argv[2], "first_top", NULL, "texture");
-    compose_images(argv[2], "first_bottom", NULL, "alpha");
-    compose_images(argv[2], "first_bottom", NULL, "white");
-    compose_images(argv[2], "first_bottom", NULL, "blue");
-    compose_images(argv[2], "first_bottom", NULL, "texture");
-    compose_images(argv[2], "disjoint_over", NULL, "alpha");
-    compose_images(argv[2], "disjoint_over", NULL, "white");
-    compose_images(argv[2], "disjoint_over", NULL, "blue");
-    compose_images(argv[2], "disjoint_over", NULL, "texture");
-    compose_images(argv[2], "disjoint_under", NULL, "alpha");
-    compose_images(argv[2], "disjoint_under", NULL, "white");
-    compose_images(argv[2], "disjoint_under", NULL, "blue");
-    compose_images(argv[2], "disjoint_under", NULL, "texture");
-    compose_images(argv[2], "disjoint_debug", NULL, "alpha");
-    compose_images(argv[2], "disjoint_debug", NULL, "white");
-    compose_images(argv[2], "disjoint_debug", NULL, "blue");
-    compose_images(argv[2], "disjoint_debug", NULL, "texture");
+    VALIDATE(compose_images(argv[2], "alpha", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "alpha", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "alpha", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "alpha", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "multiplicative", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "multiplicative", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "multiplicative", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "multiplicative", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "source_over", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "source_over", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "source_over", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "source_over", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "destination_over", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "destination_over", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "destination_over", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "destination_over", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "first_top", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "first_top", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "first_top", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "first_top", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "first_bottom", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "first_bottom", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "first_bottom", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "first_bottom", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "disjoint_over", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "disjoint_over", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "disjoint_over", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "disjoint_over", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "disjoint_under", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "disjoint_under", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "disjoint_under", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "disjoint_under", NULL, "texture"));
+    VALIDATE(compose_images(argv[2], "disjoint_debug", NULL, "alpha"));
+    VALIDATE(compose_images(argv[2], "disjoint_debug", NULL, "white"));
+    VALIDATE(compose_images(argv[2], "disjoint_debug", NULL, "blue"));
+    VALIDATE(compose_images(argv[2], "disjoint_debug", NULL, "texture"));
 
     NORMAL;
 }
@@ -756,10 +766,10 @@ ERROR_T pconvert(int argc, char **argv) {
 
     if(argc != 4) { RAISE_M("Usage: pconvert convert <file_in> <file_out>"); }
 
-    read_png(argv[2], FALSE, &image);
-    process_image(&image);
-    write_png(&image, FALSE, argv[3]);
-    release_image(&image);
+    VALIDATE(read_png(argv[2], FALSE, &image));
+    VALIDATE(process_image(&image));
+    VALIDATE(write_png(&image, FALSE, argv[3]));
+    VALIDATE(release_image(&image));
 
     NORMAL;
 }
@@ -770,47 +780,49 @@ float pbenchmark_algorithm(
     params *params,
     char *background,
     int compression,
-    int filter
+    int filter,
+    char use_opencl
 ) {
     float start_time;
     float end_time;
     float time_elapsed;
     start_time = (float) clock() / CLOCKS_PER_SEC;
-    compose_images_extra(base_path, algorithm, params, background, compression, filter, FALSE);
+    EXCEPT_P(compose_images_extra(base_path, algorithm, params, background, compression, filter, use_opencl));
     end_time = (float) clock() / CLOCKS_PER_SEC;
     time_elapsed = end_time - start_time;
     return time_elapsed;
 }
 
 ERROR_T pbenchmark(int argc, char **argv) {
+    #define ALGORITHMS_SIZE 5
+    #define COMPRESSION_SIZE 3
+    size_t index, index_j, index_k;
     float time;
-    FILE *file;
+    char *algorithms[ALGORITHMS_SIZE] = { "multiplicative", "source_over", "alpha", "disjoint_over", "disjoint_under" };
+    int compression[COMPRESSION_SIZE] = { Z_NO_COMPRESSION, Z_BEST_SPEED, Z_BEST_COMPRESSION };
+    char *compression_s[COMPRESSION_SIZE] = { "Z_NO_COMPRESSION", "Z_BEST_SPEED", "Z_BEST_COMPRESSION" };
+
+#ifdef PCONVERT_OPENCL
+    #define OPENCL_SIZE 2
+    char use_opencl[OPENCL_SIZE] = { FALSE, TRUE };
+    char *use_opencl_s[OPENCL_SIZE] = { "CPU", "OPENCL" };
+#else
+    #define OPENCL_SIZE 1
+    char use_opencl[OPENCL_SIZE] = { FALSE };
+    char *use_opencl_s[OPENCL_SIZE] = { "CPU" };
+#endif
 
     if(argc != 3) { RAISE_M("Usage: pconvert benchmark <directory>"); }
 
-    file = fopen("benchmark.txt", "wb");
-
-    time = pbenchmark_algorithm(argv[2], "source_over", NULL, "alpha", Z_NO_COMPRESSION, 0);
-    fprintf(file, "source_over Z_NO_COMPRESSION: %f\n", time);
-
-    time = pbenchmark_algorithm(argv[2], "source_over", NULL, "alpha", Z_BEST_SPEED, 0);
-    fprintf(file, "source_over Z_BEST_SPEED: %f\n", time);
-
-    time = pbenchmark_algorithm(argv[2], "source_over", NULL, "alpha", Z_BEST_COMPRESSION, 0);
-    fprintf(file, "source_over Z_BEST_COMPRESSION: %f\n", time);
-
-    fprintf(file, "\n");
-
-    time = pbenchmark_algorithm(argv[2], "multiplicative", NULL, "alpha", Z_NO_COMPRESSION, 0);
-    fprintf(file, "multiplicative Z_NO_COMPRESSION: %f\n", time);
-
-    time = pbenchmark_algorithm(argv[2], "multiplicative", NULL, "alpha", Z_BEST_SPEED, 0);
-    fprintf(file, "multiplicative Z_BEST_SPEED: %f\n", time);
-
-    time = pbenchmark_algorithm(argv[2], "multiplicative", NULL, "alpha", Z_BEST_COMPRESSION, 0);
-    fprintf(file, "multiplicative Z_BEST_COMPRESSION: %f\n", time);
-
-    fclose(file);
+    for(index = 0; index < ALGORITHMS_SIZE; index++) {
+        for(index_j = 0; index_j < COMPRESSION_SIZE; index_j++) {
+            for(index_k = 0; index_k < OPENCL_SIZE; index_k++) {
+                time = pbenchmark_algorithm(argv[2], algorithms[index], NULL, "alpha", compression[index_j], 0, use_opencl[index_k]);
+                printf("%s %s %s: %0.2f ms\n", algorithms[index], compression_s[index_j], use_opencl_s[index_k], time * 1000);
+            }
+        }
+        printf("\n");
+    }
 
     NORMAL;
 }
@@ -825,21 +837,21 @@ ERROR_T popencl(int argc, char **argv) {
 
     time_elapsed_cpu = 0;
     start_time = (float) clock() / CLOCKS_PER_SEC;
-    compose_images_extra(argv[2], "multiplicative", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE);
-    compose_images_extra(argv[2], "source_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE);
-    compose_images_extra(argv[2], "alpha", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE);
-    compose_images_extra(argv[2], "disjoint_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE);
-    compose_images_extra(argv[2], "disjoint_under", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE);
+    VALIDATE(compose_images_extra(argv[2], "multiplicative", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE));
+    VALIDATE(compose_images_extra(argv[2], "source_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE));
+    VALIDATE(compose_images_extra(argv[2], "alpha", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE));
+    VALIDATE(compose_images_extra(argv[2], "disjoint_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE));
+    VALIDATE(compose_images_extra(argv[2], "disjoint_under", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, FALSE));
     end_time = (float) clock() / CLOCKS_PER_SEC;
     time_elapsed_cpu = end_time - start_time;
 
     time_elapsed_gpu = 0;
     start_time = (float) clock() / CLOCKS_PER_SEC;
-    compose_images_extra(argv[2], "multiplicative", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE);
-    compose_images_extra(argv[2], "source_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE);
-    compose_images_extra(argv[2], "alpha", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE);
-    compose_images_extra(argv[2], "disjoint_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE);
-    compose_images_extra(argv[2], "disjoint_under", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE);
+    VALIDATE(compose_images_extra(argv[2], "multiplicative", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE));
+    VALIDATE(compose_images_extra(argv[2], "source_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE));
+    VALIDATE(compose_images_extra(argv[2], "alpha", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE));
+    VALIDATE(compose_images_extra(argv[2], "disjoint_over", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE));
+    VALIDATE(compose_images_extra(argv[2], "disjoint_under", NULL, "alpha", Z_BEST_SPEED, PNG_FILTER_NONE, TRUE));
     end_time = (float) clock() / CLOCKS_PER_SEC;
     time_elapsed_gpu = end_time - start_time;
 
@@ -852,14 +864,15 @@ ERROR_T pversion(int argc, char **argv) {
     char libpng_version_s[16];
     libpng_version(libpng_version_s);
     printf(
-        "P(NG)Convert %s (%s %s) [%s %s %d bit] [libpng %s] \n",
+        "P(NG)Convert %s (%s %s) [%s %s %d bit] [libpng %s] [%s] \n",
         PCONVERT_VERSION,
         PCONVERT_COMPILATION_DATE,
         PCONVERT_COMPILATION_TIME,
         PCONVERT_COMPILER,
         PCONVERT_COMPILER_VERSION_STRING,
         (int) PCONVERT_PLATFORM_CPU_BITS,
-        libpng_version_s
+        libpng_version_s,
+        PCONVERT_FEATURES
     );
     printf("Copyright (c) 2008-2020 Hive Solutions Lda. All rights reserved.\n");
     NORMAL;
